@@ -169,6 +169,7 @@ wbgee <- function(formula, data, id = NULL, wave = NULL,
   fit <- geepack::geeglm(fin_formula, data = data, weights = weights, 
                          offset = offset, id = id, waves = wave,
                          corstr = cor.str[1], family = family, ...)
+  gee_terms <- stats::terms(fit)
   
   ints <- e$cross_ints
   
@@ -185,7 +186,7 @@ wbgee <- function(formula, data, id = NULL, wave = NULL,
   #   data <- data[names(data) %nin% wave]
   # } 
   fit$frame <- as.data.frame(data)
-  attr(fit$frame, "terms") <- terms 
+  attr(fit$frame, "terms") <- gee_terms
   attr(fit$frame, "formula") <- formula(fit)  
   
   if (calc.fit.stats == TRUE) {
@@ -208,6 +209,7 @@ wbgee <- function(formula, data, id = NULL, wave = NULL,
                         qic = qics["QIC"], qicu = qics["QICu"],
                         cic = qics["CIC"], cor.str = cor.str[1],
                         alpha = fit_sum$corr, 
+                        interactions = c(e$within_ints, ints),
                         interaction.style = interaction.style)
   
   fit$call <- the_call
@@ -272,9 +274,9 @@ summary.wbgee <- function(object, ...) {
                         cic = x2$cic, model = est_name)
   
   coefs <- x$coefs
-  rownames(coefs) <- gsub("`", "", rownames(coefs), fixed = TRUE)
+  rownames(coefs) <- un_bt(rownames(coefs))
   if (!is.null(x2$ints)) {
-    x2$ints <- gsub("`", "", x2$ints, fixed = TRUE)
+    x2$ints <- un_bt(x2$ints)
   }
   
   varying <- x2$varying
@@ -603,4 +605,4 @@ predict_gee <- function(model, .vcov = vcov(model), newdata = NULL,
   
   return(list(fit = fit, se.fit = se.fit))
   
-}
\ No newline at end of file
+}
