@@ -1,11 +1,19 @@
 ## ----setup, include = FALSE---------------------------------------------------
+required <- c("clubSandwich", "geepack")
+do_eval <- all(sapply(required, requireNamespace, quietly = TRUE))
 knitr::opts_chunk$set(
   collapse = FALSE,
   comment = "",
-  message = FALSE
+  message = FALSE,
+  eval = do_eval
 )
 
-## -----------------------------------------------------------------------------
+## ----include = FALSE----------------------------------------------------------
+library(panelr)
+data("teen_poverty")
+teen_poverty
+
+## ----echo = FALSE-------------------------------------------------------------
 library(panelr)
 data("teen_poverty")
 teen_poverty
@@ -18,6 +26,15 @@ teen
 data("WageData")
 wages <- panel_data(WageData, id = id, wave = t)
 
+## ----eval = do_eval-----------------------------------------------------------
+library(panelr)
+library(splines)
+data("WageData")
+wages <- panel_data(WageData, id = id, wave = t)
+
+mod_spline <- wbm(lwage ~ ns(exp, df = 3) | blk, data = wages)
+mod_spline
+
 ## -----------------------------------------------------------------------------
 model <- wbm(lwage ~ wks + union + ms + occ | blk + fem, data = wages)
 summary(model)
